/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is WepPageDump.
 *
 * The Initial Developer of the Original Code is Bernhard Pollak.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */      
 
// --------------------------------------------------------------------------------
// "WebPageDump" Firefox Extension        
// --------------------------------------------------------------------------------
// - File: "overlay.js" -
// - Description:
//   UI and Controlling
// - Using:
//   * "webPageDumpUI": makes a local copy of the current webpage (incl. UI)
//   * "webPageDump": makes a local copy of the current webpage (without UI)
//   * "webPageDumpClose": save current webpage and close window (without UI)
//   * "webPageDumpBatch": save many web pages using url batch file (without UI)
//   * "webPageDumpDir": do the processing from a local source dir (without UI)
//   * "webPageDumpTest": report filesizes inside screenshot subdir (without UI)
//   * "webPageDumpCollect": looks for screenshots inside a wpd directory renames
//     and copies them to a new directory
// --------------------------------------------------------------------------------

var WPD_SCREENSHOTSUBDIR = "screenshot";          
var WPD_SCREENSHOTEXT = "png";  
var WPD_HTMLFILENAME="index.html";        
var WPD_TESTFILENAME="wpd_testresult";   // without ext 
var WPD_LOCALNAME="localhost";
var WPD_ERRORLOG="wpd_error.log";
var WPD_IDSEPERATOR="_";
var WPD_COUNTSEPERATOR="c";
var WPD_VERSIONSEPERATOR=".";


var gBatchList = [];                         
var gBatchCounter = 0;
var gDestPath = "";        
var gScreenShot = false;  
var gSkipExisting = false;    
var gSaveMaxScriptRunTime = 5;      
var gFileResult = null;
var gNoDump = false;
var gSrcDstEqual = false;               
var gDump = "wpd";  // for testing purposes... (possible: wpd, firefox, scrapbook)

// GUI HELPER FUNCTIONS

// if the errorlist is not empty, show an error dialog
function wpdShowError()
{
	if (wpdCommon.errList!="") {
		if (wpdCommon.errCount>=WPD_MAXUIERRORCOUNT) {
	    alert(wpdCommon.errCount+" ERRORS ENCOUNTED (10 displayed, multiple errors reported once)\n"+wpdCommon.errList);   
	  } else {
	  	alert(wpdCommon.errCount+" ERRORS ENCOUNTED (multiple errors reported once)\n"+wpdCommon.errList);   
	  }  	    
    return true;
	}
  return false;
}

// show a save dialog
function wpdShowSaveAsDialog()
{
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Select a File", nsIFilePicker.modeSave);
  fp.defaultString = WPD_HTMLFILENAME;
  fp.appendFilters(nsIFilePicker.filterHTML);
  fp.appendFilters(nsIFilePicker.filterAll);
  var res = fp.show();                                                         
  // selected "OK" or replace file "yes"
  if ((res == nsIFilePicker.returnOK) || (res == nsIFilePicker.returnReplace)) {   
    return fp.file;
  }                                  
  return null;
}      

// show a select directory dialog
function wpdShowSelectDirDialog()
{
   var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
   fp.init(window, "Select the target directory (a sub directory will be created automatically)", fp.modeGetFolder);

   // you can alse use 'persistentDescriptor' instead of 'path' to retrieve path string  
   var res = fp.show();
   if (res == fp.returnOK) {  
     return fp.file;
   }                                  
   return null;   
}

function wpdShowResult()
{
  if (wpdShowError()==false) 
    alert("WebPageDump: HTML Page saved sucessfully!");    	
}

// MISC HELPER FUNCTIONS    

function wpdWriteErrorLog(aPath) {        
  if (wpdCommon.errList!="") {                   
    aPath=wpdCommon.appendFilePath(wpdCommon.getFilePath(aPath),WPD_ERRORLOG);
    wpdCommon.writeFile(wpdCommon.errList,aPath);
    wpdCommon.errList="";                  
    wpdCommon.errCount=0;
  }  
}

function wpdSetEndlessScriptRunTime() {
  // we need to set the maximum script time to avoid the
  // "unresponsive script" dialog
  gSaveMaxScriptRunTime=wpdCommon.getIntPrefs("dom.max_script_run_time");  // save actual value (default is 5)
  wpdCommon.setIntPrefs("dom.max_script_run_time",0);  // so we have endless execution time 
}

function wpdRestoreScriptRunTime() {
  wpdRestore();
  if (gSaveMaxScriptRunTime==0) gSaveMaxScriptRunTime=5;  // set to default value... 
  wpdCommon.setIntPrefs("dom.max_script_run_time",gSaveMaxScriptRunTime);    
} 

function wpdInit() {   
  //wpdCommon.sizeWindow(WPD_DEFAULTWIDTH,WPD_DEFAULTHEIGHT);       
         
  // some development settings...
  wpdCommon.setBoolPrefs("javascript.options.showInConsole",true); // we want to see errors in chrome files
  wpdCommon.setBoolPrefs("nglayout.debug.disable_xul_cache",true); // no XUL caching (for changes)...
  wpdCommon.setBoolPrefs("browser.dom.window.dump.enabled",true);  // we want to see the dump messages...
  wpdCommon.setBoolPrefs("javascript.options.strict",true);        // strict javascript warnings in console...
                                                                
  // disable some animations...                                                              
  wpdCommon.setBoolPrefs("browser.blink_allowed",false);        
  wpdCommon.setBoolPrefs("browser.display.enable_marquee",false);      
  wpdCommon.setStrPrefs("image.animation_mode","none");           
  wpdCommon.setBoolPrefs("dom.disable_image_src_set",true);           
  //wpdCommon.setStrPrefs("capability.policy.default.Window.setTimeout", "noAccess");

  wpdCommon.errList="";
	wpdCommon.errCount=0;  
	gNoDump=false;    
	gSrcDstEqual =false;
	//gDumpFirefox = false;
	gScreenShot = ""; 
	gDestPath = "";
	gWaitForPaint=false;    
}                                           

function wpdRestore() {
  wpdCommon.setBoolPrefs("browser.blink_allowed",true);      
  wpdCommon.setBoolPrefs("browser.display.enable_marquee",true);      
  wpdCommon.setStrPrefs("image.animation_mode","normal");           
  wpdCommon.setBoolPrefs("dom.disable_image_src_set",false);                             
  //wpdCommon.setStrPrefs("capability.policy.default.Window.setTimeout", "");
}




// NAMING FUNCTIONS
                                                         
// naming specification
// format: ABC_XXXXcC.V or ABC_XXXX.V
//
//       name     type     req/opt   description
// --------------------------------------------------------------------------------
// ABC   domain   string   required  domain identifier (domain name or "localhost")
// XXXX  id       integer  required  sum of all URL chars(4 chars, leading zeros)
// C     counter  integer  optional  indicates different URLs with same IDs)
// V     version  integer  required  indicates same URL but different version)

                                                            
// adds all charcodes        
function wpdGetQuersumme(aText) {
  var c=0;
  try {
    for (var i=0;i<aText.length;i++) 
      c=c+aText.charCodeAt(i);
  } catch(ex) {  
  }
  return c;
}

// generate a directory name from the url using
// the domain and a counter build from the charcodes
function wpdGenerateNameFromUrl(aURI) {
  aURI=wpdCommon.convertUrlToASCII(aURI);    // no exotic chars - just ASCII
  var domainName=wpdCommon.getHostName(aURI);    
  var fileNumber=wpdGetQuersumme(aURI) % 10000; // adding all charcodes modulo 10000
  var fileName=WPD_LOCALNAME;                 // for local files
  if (domainName!="") {
    fileName=wpdCommon.getValidFileName(domainName); 
  }                                                                             
  // return the new name with the id and version "0"           
  return fileName.replace(/[.]/g,"_")+WPD_IDSEPERATOR+wpdCommon.addLeftZeros(fileNumber,4)+WPD_VERSIONSEPERATOR+"0";  
}          
      
// split a directory name in id, counter and version      
function wpdGetDirComponents(dir)
{                                                                                       
  var ret=[]; 
  ret["name"] = (pos = dir.lastIndexOf(WPD_IDSEPERATOR)) != -1 ? dir.substring(0,pos) : dir;  
  var comp = (pos = dir.lastIndexOf(WPD_IDSEPERATOR)) != -1 ? dir.substring(++pos) : ""; 
  // we let be "id" a string because of leading zeros...
  ret["id"] = (pos = comp.lastIndexOf(WPD_VERSIONSEPERATOR)) != -1 ? comp.substring(0,pos) : comp; 
  ret["id"] = (pos = comp.lastIndexOf(WPD_COUNTSEPERATOR)) != -1 ? comp.substring(0,pos) : ret["id"];
  ret["count"] = (pos = comp.lastIndexOf(WPD_COUNTSEPERATOR)) != -1 ? comp.substring(++pos) : 0;  
  if (ret["count"]!=0) 
    ret["count"] = parseInt((pos = ret["count"].lastIndexOf(WPD_VERSIONSEPERATOR)) != -1  ? ret["count"].substring(0,pos) : ret["count"]);
  ret["version"] = (pos = comp.lastIndexOf(WPD_VERSIONSEPERATOR)) != -1 ? parseInt(comp.substring(++pos)) : 0; 
  dump("[wpdGetDirComponents] dir: "+dir+" Components: "+ret["id"]+","+ret["count"]+","+ret["version"]+"\n");
  return ret;  
}                             
                                                      
// increment the version number  - aPath is a fully qualified path
function wpdIncDirVersion(aPath)
{        
  path=wpdCommon.getFilePath(aPath);
  dir=wpdCommon.getFileLeafName(aPath);
  comp=wpdGetDirComponents(dir);           
  dir=comp["name"]+WPD_IDSEPERATOR+comp["id"];                                
  // if a counter is present we have to add it...
  if (comp["count"]>0) dir=dir+WPD_COUNTSEPERATOR+comp["count"];
  dir=dir+WPD_VERSIONSEPERATOR+(comp["version"]+1);
  return wpdCommon.appendFilePath(path,dir);
}    

// increment the counter - aPath is a fully qualified path
function wpdIncDirCounter(aPath)
{
  path=wpdCommon.getFilePath(aPath);
  dir=wpdCommon.getFileLeafName(aPath);
  comp=wpdGetDirComponents(dir);           
  dir=comp["name"]+WPD_IDSEPERATOR+comp["id"];       
  dir=dir+WPD_COUNTSEPERATOR+(comp["count"]+1);
  dir=dir+WPD_VERSIONSEPERATOR+comp["version"]; 
  return wpdCommon.appendFilePath(path,dir);  
}

// get a unique directory name
function wpdGetUniqueDir(aURI,aPath) {
  if (wpdCommon.pathExists(aPath)) {   
    aURL=wpdDOMSaver.getMetaBaseURL(wpdCommon.appendFilePath(aPath,WPD_HTMLFILENAME));     
    if (aURI==aURL)  {
      // Same URL -> increment version
      while (wpdCommon.pathExists(aPath) && !gSkipExisting) {
        aPath=wpdIncDirVersion(aPath);
      }       
      return aPath;
    } else { 
      // we have to call wpdGetUniqueDir another time
      // cause this is a different url and maybe
      // another counter has different versions 
      return wpdGetUniqueDir(aURI,wpdIncDirCounter(aPath));
    }                                          
  } 
  //dump("[wpdGetUniqueDir] first directory...\n");          
  return aPath;
}             

function wpdGetDirName(aURI,aDestPath) {               
 if (wpdCommon.trim(aURI)=="") return "";       
 var dirName=wpdGenerateNameFromUrl(aURI);                   
 return wpdCommon.appendFilePath(aDestPath,dirName);  
}                                              

function wpdGetUniqueDirName(aURI,aDestPath) {
  if (wpdCommon.trim(aURI)=="") return "";        		
  var filePath=wpdGetDirName(aURI,aDestPath);
  dump("[wpdGetUniqueDirName]  init path: "+filePath+"\n");          
  filePath=wpdGetUniqueDir(aURI,filePath);                                                    
  dump("[wpdGetUniqueDirName] final path: "+filePath+"\n"); 
  return filePath;    
}      

function wpdGetUniqueFileName(aURI,aDestPath) {
  try {
    if (wpdCommon.trim(aURI)=="") return "";   
    
    var ext = wpdCommon.splitFileName(wpdCommon.getFileLeafName(aDestPath))[1];
    if ((ext!="") && ((ext=="html") || (ext=="htm"))) return aDestPath;
            
    return wpdCommon.appendFilePath(wpdGetUniqueDirName(aURI,aDestPath),WPD_HTMLFILENAME);
  } catch(ex) {
    dump("EXCEPTION "+ex+"\n");
  }
}

function wpdMkDirGetFileName(aDestFile) {  
  if (!MODE_SIMULATE) wpdCommon.createDir(wpdCommon.getFilePath(aDestFile));    
  return aDestFile;
}

/*function wpdMkDirAndGetName(aURI,aDestPath) {
  filePath = wpdGetUniqueDirName(aURI,aDestPath);
  if (!MODE_SIMULATE) wpdCommon.createDir(filePath);    
  return wpdCommon.appendFilePath(filePath,WPD_HTMLFILENAME);
}*/
      


// UI PROCESSING FUNCTION

function wpdProcessUI(){     
  if (gFileResult) {                      
    wpdSetEndlessScriptRunTime();           
    // var filepath=wpdCommon.getFilePath(gFileResult.path); // only directory          
    var filepath=gFileResult.path; 
    var aDestFile=wpdGetUniqueFileName(wpdCommon.getURL(),filepath);
    webPageDump(wpdMkDirGetFileName(aDestFile),false);
    // wpdShowResult();      
    wpdWriteErrorLog(aDestFile);        
    wpdRestoreScriptRunTime();        
  }
}       


// SCREENSHOT FUNCTION  
                                           
function wpdScreenShotExists(aDestFile) {
 var tempDir = wpdCommon.appendFilePath(wpdCommon.getFilePath(aDestFile),WPD_SCREENSHOTSUBDIR);           
 tempDir = wpdCommon.appendFilePath(tempDir,gScreenShot+"."+WPD_SCREENSHOTEXT);    
 return wpdCommon.pathExists(tempDir);
}
                             
// we are calling the pearlcrescent pagesaver extension
// has to be installed for using this functionality
function wpdScreenShot(aDestFile){     
  if (gScreenShot!="") {    
    dump("[wpdScreenShot] "+aDestFile+"\n");                                       
    try {
      // this is only for checking that pearlcrescent is present
      FilenameFromPageTitle("");                                                                        
      var tempDir = wpdCommon.appendFilePath(wpdCommon.getFilePath(aDestFile),WPD_SCREENSHOTSUBDIR);             
      if (!gSkipExisting || !wpdCommon.pathExists(wpdCommon.appendFilePath(tempDir,gScreenShot+"."+WPD_SCREENSHOTEXT)) ) {
        wpdCommon.sizeWindow(WPD_DEFAULTWIDTH,WPD_DEFAULTHEIGHT);       
        wpdCommon.createDir(tempDir);          
        // we set the browser dir pref, cause this is used by pagesaver as dest directory
        var sBrowserDir="";
        try {
          sBrowserDir=wpdCommon.getStrPrefsEx("browser.download.dir");
        } catch(e) {}
        wpdCommon.setStrPrefsEx("browser.download.dir",tempDir);
        var sTitle = top.window._content.document.title;
        // doc title is used as filename     
        top.window._content.document.title=gScreenShot;    
                                                                           
        dump("[wpdScreenShot] creating screenshot...\n"); 
        try {                                               
          // save image with entire page, commandline and tempdir as target
          // unfortunately tempdir is ignored, so we have to hack around
          // we need also commandline "true" so that there is no dialog showed
          SavePageImage(true, true, tempDir);                 
          // little hack, cause we set commandline previously "true" which
          // result in setting the pageSaverCloseWhenDone also "true"
          // pagesaver has a little delay for the saveimage function call
          // so we set the close property back to "false" and prevent
          // the extension from closing our window...
          window.pageSaverCloseWhenDone=false;
        } catch(e) {}
        
        // set the hacks back to the previous values...  
        wpdCommon.setStrPrefsEx("browser.download.dir",sBrowserDir);                 
        top.window._content.document.title=sTitle;      
      }
                
    } catch(ex) {
      dump("[wpdScreenShot] EXCEPTION: "+ex+"\n");
    }
  }                                    
}

        
// MAIN DUMP FUNCTIONS
      
// save the current open webpage without User Interface	
// (needs the destination path as argument)
function webPageDump(aDestFile,aTimeOut){             
  dump("[webPageDump]...\n");     
  if (!wpdIsTimedOut()) {       
    if (!gNoDump) {   
      if (!gSkipExisting || !wpdCommon.pathExists(aDestFile)) {
        if (gDump=="firefox") {
          wpdCommon.saveWebPage(aDestFile);
        } else if (gDump=="scrapbook") {        
          sbContentSaver.init();                                			    
			    var obj_File = Components.classes["@mozilla.org/file/local;1"].getService(Components.interfaces.nsILocalFile);        
          obj_File.initWithPath(wpdCommon.getFilePath(aDestFile));    
          // set necessary vars...
          sbContentSaver.frameList = sbContentSaver.flattenFrames(top.window._content); 
          sbContentSaver.contentDir=obj_File;  
          sbContentSaver.item.chars  = top.window._content.document.characterSet;
		      sbContentSaver.item.source = top.window._content.location.href;                
          sbContentSaver.saveDocumentInternal(top.window._content.document,"index");
        } else {        
          wpdDOMSaver.init(aDestFile);
          wpdDOMSaver.saveHTMLDocument();
        }     
      }
    }  
    wpdScreenShot(aDestFile);             
  } else {
    wpdCommon.addError("[webPageDump] Could not open URL (TimeOut)\n");
  }
}                     


// BATCH PROCESSING FUNCTIONS      

function wpdGetBatchDestPath() {
  if (gSrcDstEqual) {
    return wpdCommon.trim(gBatchList[gBatchCounter]);                                    
  } else {                   
    return wpdGetUniqueFileName(wpdCommon.trim(gBatchList[gBatchCounter]),gDestPath);
  }  
}     

// checks if there was a redirecting which may reflect 
// that the requested URL is not longer present
function wpdDifferentURLs(aExpectedURL)
{
  var browser=this.top.getBrowser();                   
  var dst=aExpectedURL;
  try {
    var dst=wpdCommon.filePathToURI(dst);
  } catch(ex) {}
  if (browser.contentDocument.location!=wpdCommon.trim(dst))   {
    return browser.contentDocument.location;
  }             
  return "";
}

function wpdBatchProcess() {        
  try {        
    if ( (wpdCommon.trim(gBatchList[gBatchCounter])!="")  ) {            
      dump("[wpdBatchProcess] "+(gBatchCounter+1)+"/"+gBatchList.length+"\n");     
      var aDestFile=wpdGetBatchDestPath();        
      if (!gSkipExisting || (gSrcDstEqual && !wpdScreenShotExists(aDestFile)) || !wpdCommon.pathExists(aDestFile)) {
        if (!wpdIsTimedOut()) {            
          var s = wpdDifferentURLs(wpdCommon.trim(gBatchList[gBatchCounter]));
          if (s != "") wpdCommon.addError("[wpdBatchProcess] loaded URL is <> requested URL ("+s+"<>"+gBatchList[gBatchCounter]+"\n");
          webPageDump(wpdMkDirGetFileName(aDestFile),true);  
        } else {
          wpdCommon.addError("[wpdBatchProcess] Timeout - Could not open URL "+gBatchList[gBatchCounter]+"\n");
        }                                 
        wpdWriteErrorLog(aDestFile);       
      }
    }        
                                              
    // are we finished?                                            
    if (gBatchCounter<gBatchList.length-1) {  
      gBatchCounter++;                                                                             
      var aDestFile=wpdGetBatchDestPath();
      if ( (wpdCommon.trim(gBatchList[gBatchCounter])!="") && 
      (!gSkipExisting || (gSrcDstEqual && !wpdScreenShotExists(aDestFile)) || !wpdCommon.pathExists(aDestFile)) ) {       
        wpdLoadURL(gBatchList[gBatchCounter],"wpdBatchProcess()");  
      } else {       
        // empty line or skip existing file ...  
        dump("skipping ... \n");
        gBrowserWindow.setTimeout("wpdBatchProcess()", 100);  
      }                              
    } else {                   
      // Yes - we are finished
      wpdRestoreScriptRunTime(); 
      dump("\n\n");
      dump(" *************************************************************** \n");
      dump(" ***************************  READY  *************************** \n");                
      dump(" *************************************************************** \n");   
      wpdCommon.playSound();          
      // we cannot close the window when making screenshots otherwise 
      // the last webpage would be ignored 
      if (gScreenShot=="") close();  
    } 
  } catch(ex) {   
    dump("[wpdBatchProcess] EXCEPTION: "+ex+"\n");
    gBatchCounter++;  
    if (gBatchCounter<gBatchList.length) gBrowserWindow.setTimeout("wpdBatchProcess()", 100);  
  }
}   


// INTERFACES

// save the current open webpage with User Interface
function webPageDumpUI(){  
  try {   
    //update();
    wpdInit(); 
    dump("\n[webPageDumpUI]\n");   
    // we want a screenshot temporary 
    //gScreenShot="online";
    gFileResult=wpdShowSelectDirDialog();                 
    // we need a timeout for the repaint of the SaveAs Dialog
    wpdGetTopBrowserWindow().setTimeout(wpdProcessUI, 100);     
  } catch (ex) {
    dump("EXCEPTION"+ex+"\n");
  }    
}            

// save the current open webpage without User Interface
// and close the actual window	
// (needs the destination path as argument).
// If aScreenshot is not empty a call to the 
// pearlcrescent pagesaver is made to generate a screenshot
// of the whole webpage inside the WPD_SCREENSHOTSUBDIR
// subdirectory using aScreenshot as filename
function webPageDumpClose(aDestPath,aScreenShot){       
  try {
    wpdInit();               
    gScreenShot=aScreenShot;
    if (gScreenShot!="") gWaitForPaint=true;
    dump("\n[webPageDumpClose]\n");     
    wpdSetEndlessScriptRunTime();       
    webPageDump(wpdGetUniqueFileName(wpdCommon.getURL(),aDestPath),false);  
    wpdWriteErrorLog(aDestPath);
    wpdRestoreScriptRunTime();      
    wpdGetTopBrowserWindow().close();
  } catch (ex) {
    dump("EXCEPTION"+ex+"\n");
  }
}                                                          

// do the batch processing with the batch file
// and close the actual window afterwards.
// If aScreenshot is not empty a call to the 
// pearlcrescent pagesaver is made to generate a screenshot
// of the whole webpage inside the WPD_SCREENSHOTSUBDIR
// subdirectory using aScreenshot as filename
function webPageDumpBatch(aBatchFile,aDestPath,aScreenShot,aSkipExisting){      
  try {                
    wpdInit();                
    dump("\n[webPageDumpBatch]\n");        
    wpdSetEndlessScriptRunTime();
    gBatchList=wpdCommon.readFile(wpdCommon.URLToFilePath(aBatchFile),true,true);
    gScreenShot=aScreenShot;       
    gSkipExisting=aSkipExisting;
               
    if (gScreenShot!="") gWaitForPaint=true;
    if (aDestPath=="") {           
      aDestPath = wpdCommon.getFilePath(aBatchFile);
    }
    gDestPath = aDestPath; 
    gBatchCounter = 0;   
        
    if (gBatchList.length>0) {            
      while (wpdCommon.trim(gBatchList[gBatchCounter]) == "" ) {
        gBatchCounter++;
      }
      wpdLoadURL(gBatchList[gBatchCounter],"wpdBatchProcess()");  
    }
  } catch(ex) {   
    dump("[webPageDumpBatch] EXCEPTION: "+ex+"\n");
  }
}     

// do the processing from local aSourceDir to the
// directory aDestPath and close actual window afterwards.    
// If aScreenshot is not empty a call to the 
// pearlcrescent pagesaver is made to generate a screenshot
// of the whole webpage inside the WPD_SCREENSHOTSUBDIR
// subdirectory using aScreenshot as filename
function webPageDumpDir(aSourcePath,aDestPath,aScreenShot,aSkipExisting) {
  try {                
    wpdInit();            
    dump("\n[webPageDumpDir]\n");        
    wpdSetEndlessScriptRunTime();                                       
    
    gScreenShot = aScreenShot;
    gDestPath = aDestPath;      
    gSkipExisting=aSkipExisting;            
    
    if (gDestPath=="") gDestPath=aSourcePath;
     // no dump if source=dest
    if (aSourcePath==gDestPath) {
      gNoDump=true;      // we dont want a dump 
      gSrcDstEqual=true; // we want to use batchlist as destination
    }    
    if (gScreenShot!="") gWaitForPaint=true;               
    
    // Fill gBatchList with directory entries    
    var files = wpdCommon.readDir(aSourcePath);  
    dump("[webPageDumpDir] processing files...\n");
    while(files.hasMoreElements()){    
      file=files.getNext().QueryInterface(Components.interfaces.nsILocalFile);     
      if (file.isDirectory()) {
        aPath=file.path;
        aPath=wpdCommon.appendFilePath(aPath,WPD_HTMLFILENAME);
        if (wpdCommon.pathExists(aPath)) {         
          gBatchList.push(aPath);    
        }   
      }                  
    }   
    
    gBatchCounter = 0;   
    if (gBatchList.length>0)
      wpdLoadURL(gBatchList[gBatchCounter],"wpdBatchProcess()");  
      
  } catch(ex) {   
    dump("[webPageDumpDir] EXCEPTION: "+ex+"\n");
  }  
}         

// looks for screenshots inside the subdirectories of aTestPath
// and writes a WPD_TESTFILENAME_XXXX.txt inside aTestPath where
// the size of the screenshots together with the subdirectory 
// name and original url is listed. The grouping is done 
// automatically using the filenames of the screenshots
function webPageDumpTest(aTestPath) { 
  try {
    dump("\n[webPageDumpTest]\n");        
    wpdSetEndlessScriptRunTime();
    var screenshotList = [];  
    var screenshots = [];
    var resultList = [];   
    var percentList = [];   
    var aFileName = "";
    var aSubDirName = "";   
    var aSubDirPath="";                               
    var aScreenshotPath="";
    var aCount=0; 
    var file=null;         
    var skippeddirs="";
    var nourldirs="";
    
    dump("[webPageDumpTest] processing subdirectories...\n");    
    
    // get subdirectory listing        
    if (!wpdCommon.pathExists(aTestPath)) {
      wpdCommon.addError("[webPageDumpTest]: "+aTestPath+" does not exist!\n"); 
      return;
    }
    var files = wpdCommon.readDir(aTestPath);      
    while(files.hasMoreElements()){   
      file=files.getNext().QueryInterface(Components.interfaces.nsILocalFile);     
      if (file.isDirectory()) {                        
        // add the screenshot path
        aSubDirPath=file.path; 
        dump("["+aCount+"] processing "+aSubDirPath+"...\n");            
        aSubDirName=wpdCommon.getFileLeafName(aSubDirPath);                             
        aScreenshotPath=wpdCommon.appendFilePath(aSubDirPath,WPD_SCREENSHOTSUBDIR);
        
        //add an entry with ID (subdirname) and original URL (from WPD_HTMLFILENAME)                       
        resultList[aCount] = new Array("ID","URL","Date","files");                   
        resultList[aCount]["ID"]=aSubDirName;
        resultList[aCount]["URL"]=wpdDOMSaver.getMetaBaseURL(wpdCommon.appendFilePath(aSubDirPath,WPD_HTMLFILENAME));
        resultList[aCount]["Date"]=wpdDOMSaver.getMetaDate(wpdCommon.appendFilePath(aSubDirPath,WPD_HTMLFILENAME));
        resultList[aCount]["files"]=[];                                                                                                                      
        // get screenshot files listing
        if (wpdCommon.pathExists(aScreenshotPath)) {  
          screenshots=wpdCommon.readDir(aScreenshotPath);                        
          while(screenshots.hasMoreElements()) {   
            screenshot=screenshots.getNext().QueryInterface(Components.interfaces.nsILocalFile);     
            if (!screenshot.isDirectory()) {                                
              aFileName=wpdCommon.getFileLeafName(screenshot.path);                                           
              if (screenshotList[aFileName]==undefined) screenshotList[aFileName]=screenshotList.length;                            
              resultList[aCount]["files"][aFileName] = new Array("Name","Size");
              resultList[aCount]["files"][aFileName]["Name"]=aFileName;
              resultList[aCount]["files"][aFileName]["Size"]=wpdCommon.fileSize(screenshot.path);
            }
          }                 
        } else {
          skippeddirs = skippeddirs+aSubDirName+"; ";
        } 
        if (resultList[aCount]["URL"]=="")  nourldirs = nourldirs+aSubDirName+"; ";
        aCount++;          
      }
    }      
        
    var cr="\n";
		var tab="\t";     
          
    // generate unique filename                                                                      
    var aDestFile=WPD_TESTFILENAME+"_0000.txt";
    var aDest=wpdCommon.appendFilePath(aTestPath,aDestFile);
    var i=1;
    while (wpdCommon.pathExists(aDest) && (i<100)) {          
      aDestFile=WPD_TESTFILENAME+"_"+wpdCommon.addLeftZeros(i,4)+".txt";
      aDest=wpdCommon.appendFilePath(aTestPath,aDestFile);
      i++;
    }         

    
    // write the file information
    var dateObj = new Date(); 
    var d = dateObj.getUTCFullYear()+"-"+dateObj.getUTCMonth()+"-"+dateObj.getUTCDate();
		d = d+"T"+dateObj.getUTCHours()+":"+dateObj.getUTCMinutes()+"Z";          
		
    var r = "webpagedump test"+cr;
    r = r + "filename:" +aDestFile+cr;
    r = r + "date: "+d+cr;
    r = r + "test dir: "+aTestPath+cr;
    r = r + "subdir count: "+aCount+cr;
    
    var d="";

    // write the header ...
    d = d + "ID" + tab + "URL" + tab + "Date" +tab; 
    for(var n in screenshotList) {
       d = d +  n + tab;     
       percentList[n]=0;   
    }
    d = d + cr;     
    
    var addit=false;
    var diffsizecount=0;
    var oldsize=-1;          
    
    
    // ...and the data
    for ( var i=0; i<resultList.length; i++ ) {   
        reference=0;
        addit=false;
        for(var n in screenshotList) {            
          if ( (resultList[i]["files"][n]!=undefined) && (resultList[i]["files"][n]["Name"]=="online.png") ) {
            reference=resultList[i]["files"][n]["Size"];
            break;
          }    
        }  
        if (reference!=0) {
          for(var n in screenshotList) {            
            if ( (resultList[i]["files"][n]!=undefined) && (resultList[i]["files"][n]["Name"]!="online.png") ) {
              if (resultList[i]["files"][n]["Size"]!=reference) 
                percentList[resultList[i]["files"][n]["Name"]]++;
            }
          }        
        }   
        if (addit) diffsizecount++;
        d = d + resultList[i]["ID"] + tab + resultList[i]["URL"] + tab + resultList[i]["Date"] +tab;
        for(var n in screenshotList) {            
          // this makes sure that the cell order is always the same
          if (resultList[i]["files"][n]!=undefined) {    
            d = d + resultList[i]["files"][n]["Size"] + tab; 
          } else {
            d = d + "" + tab; // this file is not present => empty cell
          }
        }
        d = d + cr;                                                          
    }                        
         
    r = r + "analysis: ";
    for(var n in screenshotList) {
      if (percentList[n]>0) {
         r = r+n+" ("+percentList[n]+"|";
         r = r+((percentList[n]*100)/aCount)+"%); ";
      }    
    }     
    
    r = r + cr + "no screenshots: "+skippeddirs+cr;
    r = r + "no URL: "+skippeddirs+cr+cr;
    
    wpdCommon.writeFile(r+d,aDest);
    dump("[webPageDumpTest] "+aDest+" written!\n");
    dump("[webPageDumpTest] READY!\n");
    
  } catch(ex) {   
    dump("[webPageDumpTest] EXCEPTION: "+ex+"\n");
  }  
  wpdRestoreScriptRunTime();    
}

// looks for screenshots inside the subdirectories of aTestPath
// and writes a WPD_TESTFILENAME_XXXX.txt inside aTestPath where
// the size of the screenshots together with the subdirectory 
// name and original url is listed. The grouping is done 
// automatically using the filenames of the screenshots
function webPageDumpCollect(aPath, aDest) { 
  try {
    dump("\n[webPageDumpTest]\n");        
    wpdSetEndlessScriptRunTime();
    var screenshotList = [];  
    var screenshots = [];
    var resultList = [];   
    var percentList = [];   
    var aFileName = "";
    var aSubDirName = "";   
    var aSubDirPath="";                               
    var aScreenshotPath="";
    var aCount=0; 
    var file=null;         
    var skippeddirs="";
    var nourldirs="";
    
    
    
    if (aPath=="") {
      wpdCommon.addError("[webPageDumpCollect]: Source path is empty\n"); 
      return;    
    }
    
    if (aDest=="") {
      aDest=wpdCommon.appendFilePath(aPath,"wpd_collect");
      wpdCommon.createDir(aDest);
    }
    
    if (aPath==aDest) {
      wpdCommon.addError("[webPageDumpCollect]: "+aPath+" == "+aDest+"\n"); 
      return;    
    }    
   
    
    // get subdirectory listing        
    if (!wpdCommon.pathExists(aPath)) {
      wpdCommon.addError("[webPageDumpCollect]: Source path ("+aPath+") does not exist!\n"); 
      return;
    }
    
    if (!wpdCommon.pathExists(aDest)) {
      wpdCommon.addError("[webPageDumpCollect]: Destination path ("+aDest+") does not exist!\n"); 
      return;
    }
    
    dump("[webPageDumpCollect] processing subdirectories...\n");        
    var files = wpdCommon.readDir(aPath);      
    while(files.hasMoreElements()){   
      file=files.getNext().QueryInterface(Components.interfaces.nsILocalFile);     
      if (file.isDirectory()) {                        
        // add the screenshot path
        aSubDirPath=file.path; 
        dump("["+aCount+"] processing "+aSubDirPath+"...\n");            
        aSubDirName=wpdCommon.getFileLeafName(aSubDirPath);                             
        aScreenshotPath=wpdCommon.appendFilePath(aSubDirPath,WPD_SCREENSHOTSUBDIR);                                                                                                                
        // get screenshot files listing
        if (wpdCommon.pathExists(aScreenshotPath)) {  
          screenshots=wpdCommon.readDir(aScreenshotPath);
          var aScreenshots=0;    
          while(screenshots.hasMoreElements()) {   
            screenshot=screenshots.getNext().QueryInterface(Components.interfaces.nsILocalFile);     
            if (!screenshot.isDirectory()) {                                
              aFileName=wpdCommon.getFileLeafName(screenshot.path);    
              var res=wpdCommon.splitFileName(aFileName);
              var del=".";   
              if (res[1]=="") del="";
              if (aScreenshots>0) res[1]="_"+aScreenshots+del+res[1];
              var aDestFilePath=wpdCommon.appendFilePath(aDest,aSubDirName)+del+res[1];                        
              wpdCommon.copyFile(screenshot.path,aDestFilePath);                                        
              aScreenshots++;              
            }
          }   
          
        } else {
          skippeddirs = skippeddirs+aSubDirName+"; ";
        } 
        aCount++;          
      }
    }      
        
 
    dump("[webPageDumpCollect] "+aCount+" Directories processed!\n");
    dump("[webPageDumpCollect] READY!\n");
    
  } catch(ex) {   
    dump("[webPageDumpCollect] EXCEPTION: "+ex+"\n");
  }  
  wpdRestoreScriptRunTime();    
}
     


                          
                                                 
